
#include "FloatImage.h"

#define cimg_display 0
#include "CImg.h"
using namespace cimg_library;

FloatImage::FloatImage(const char* filename)
{
  CImg<unsigned char> img(filename);
  m_width   = img.width();
  m_height  = img.height();
  m_data.resize(m_width * m_height);

  for(int j=0; j<m_height; ++j)
    for(int i=0; i<m_width; ++i)
    {
      m_data[i+j*m_width] = Vec3(img(i,j,0,0)/255., img(i,j,0,1)/255., img(i,j,0,2)/255.);
    }
}

void FloatImage::save(const char* filename) const
{
  CImg<unsigned char> img(m_width, m_height, 1, 3);

  for(int j=0; j<m_height; ++j)
    for(int i=0; i<m_width; ++i)
    {
      img(i,j,0,0) = m_data[i+j*m_width].x*255;
      img(i,j,0,1) = m_data[i+j*m_width].y*255;
      img(i,j,0,2) = m_data[i+j*m_width].z*255;
    }

  img.save(filename);
}

FloatImageGray::FloatImageGray(int width, int height)
{
  m_width = width;
  m_height = height;
  m_data.resize(m_width * m_height, 0);
}

FloatImageGray::FloatImageGray(const char* filename)
{
  CImg<unsigned char> img(filename);
  m_width = img.width();
  m_height = img.height();
  m_data.resize(m_width * m_height);

  for(int j=0; j<m_height; ++j)
    for(int i=0; i<m_width; ++i)
      m_data[i+j*m_width] = (img(i,j,0,0)*11.0 + img(i,j,0,1) * 16.0 + img(i,j,0,2)*5.0)/(32.*255.);
}

void FloatImageGray::save(const char* filename) const
{
  CImg<unsigned char> img(m_width, m_height, 1, 1);

  for(int j=0; j<m_height; ++j)
    for(int i=0; i<m_width; ++i)
      img(i,j) = m_data[i+j*m_width]*255;

  img.save(filename);
}
