

#include "common.h"
#include <iostream>
#include <cstring>

// Declaration of the CUDA public functions:
// (they are implemented in the respective .cu files)
extern void binarize_cuda(std::vector<float>& value, float threshold);
extern void apply_3x3filter_cuda(FloatImageGray& img, float filter[3][3], int nb_passes);

// reference code to apply n times a 3x3 filter to an image
// (implemented below)
void apply_3x3filter_cpu(FloatImageGray& img, float filter[3][3]);

int main(int argc, char *argv[])
{
  if(argc!=3)
  {
    std::cout << "Usage: " << argv[0] << " <input> <output>" << std::endl;
    exit(1);
  }
  
  FloatImageGray img(argv[1]);

  /* filter 1: binarize the image */
  binarize_cuda(img.values(), 0.5);

  /* filter 2: convolution */
  {
    // a low-pass filter to be used with apply_3x3filter_*
    float filter[3][3]
      = {{0.05, 0.1, 0.05},
         {0.1,  0.4, 0.1},
         {0.05, 0.1, 0.05}};

    // Test with the reference CPU version:
    if(0)
    {
      FloatImageGray img_ref(img);
      Timer t;
      startTimer(t);
      // Apply the filter 10 times
      for(int k=0; k<20; ++k)
        apply_3x3filter_cpu(img_ref, filter);
      stopTimer(t);
      std::cout << "3x3 filtering on the CPU: " << t << "\n";
      img_ref.save("cpu_convolution.png");
    }

    if(0)
    {
      apply_3x3filter_cuda(img, filter, 20);
    }
  }
    
  img.save(argv[2]);
  
  return 0;
}

// reference code to apply a 3x3 filter to an image
void apply_3x3filter_cpu(FloatImageGray& img, float filter[3][3])
{
  using namespace std;
  int w = img.width();
  int h = img.height();
  float* tmp = new float[w*h];
  float* input  = img.raw_data();
  for(int x=0; x<w; ++x)
    for(int y=0; y<h; ++y)
    {
      float sum = 0;
      for(int i=0; i<3; ++i)
        for(int j=0; j<3; ++j)
          // clamp the pixel indices to the image boundary
          sum += input[max(0,min(x-1+i,w-1)) + max(0,min(y-1+j,h-1))*w] * filter[i][j];
      tmp[x+y*w] = sum;
    }
  memcpy(img.raw_data(), tmp, w*h*sizeof(float));
  delete[] tmp;
}